﻿using DotNetCore.CAP;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.Orders;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using Hotline.Share.Mq;
using MapsterMapper;
using MediatR;

namespace Hotline.Application.Orders.OrderScreenHandler;
public class OrderScreenNextWorkflowHandler : INotificationHandler<NextStepNotify>
{
	private readonly ICapPublisher _capPublisher;
	private readonly IMapper _mapper;
	private readonly IOrderScreenRepository _orderScreenRepository;

	public OrderScreenNextWorkflowHandler(
		ICapPublisher capPublisher,
		IMapper mapper,
		IOrderScreenRepository orderScreenRepository)
	{
		_capPublisher = capPublisher;
		_mapper = mapper;
		_orderScreenRepository = orderScreenRepository;
	}

	/// <summary>Handles a notification</summary>
	/// <param name="notification">The notification</param>
	/// <param name="cancellationToken">Cancellation token</param>
	public async Task Handle(NextStepNotify notification, CancellationToken cancellationToken)
	{
		if (notification.Workflow.ModuleCode == WorkflowModuleConsts.OrderScreen) 
		{
			var workflow = notification.Workflow;
			var nextTag = string.IsNullOrEmpty(notification.NextStepDefine.Tag)
				? null
				: System.Text.Json.JsonSerializer.Deserialize<DefinitionTag>(notification.NextStepDefine.Tag);
			var screen = await _orderScreenRepository.Queryable().Includes(x => x.Order)
				.Where(x => x.Id == workflow.ExternalId).FirstAsync(cancellationToken);
			if (screen != null)
			{
				screen.Status = EScreenStatus.Approval;
				screen.Flowed(workflow.FlowedUserIds, workflow.FlowedOrgIds, workflow.HandlerUsers, workflow.HandlerOrgs);
				//如果下个节点是省审批，则修改为省甄别
				if (nextTag is not null && nextTag.Type == TagDefaults.TagType.Org && nextTag.Value == TagDefaults.TagValue.Province)
					screen.IsProScreen = true;
				await _orderScreenRepository.UpdateAsync(screen, cancellationToken);
			}

			if (nextTag is not null && nextTag.Type == TagDefaults.TagType.Org)
			{
				switch (nextTag.Value)
				{
					case TagDefaults.TagValue.Province:
						if (screen != null)
						{
							var screenDto = _mapper.Map<OrderScreenListDto>(screen);
							if (screen.Order != null && screen.Order.Source == ESource.ProvinceStraight)
							{
								var screenOrderDto = _mapper.Map<OrderDto>(screen.Order);
								//推省上
								_capPublisher.Publish(EventNames.HotlineOrderScreenApply, new PublishScreenDto()
								{
									Order = screenOrderDto,
									Screen = screenDto,
									ClientGuid = ""
								});
							}
						}

						break;
				}
			}
		}
	}
}